#!/usr/bin/python

#   BSD LICENSE
#
#   Copyright(c) 2010-2014 Intel Corporation. All rights reserved.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     * Neither the name of Intel Corporation nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Test data for autotests

from glob import glob
from autotest_test_funcs import *

# quick and dirty function to find out number of sockets
def num_sockets():
	result = len(glob("/sys/devices/system/node/node*"))
	if result == 0:
		return 1
	return result

# Assign given number to each socket
# e.g. 32 becomes 32,32 or 32,32,32,32
def per_sockets(num):
    return ",".join([str(num)] * num_sockets())

# groups of tests that can be run in parallel
# the grouping has been found largely empirically
parallel_test_group_list = [

{
	"Prefix":	"group_1",
	"Memory" :	per_sockets(8),
	"Tests" :
	[
		{
		 "Name" :	"Timer autotest",
		 "Command" : 	"timer_autotest",
		 "Func" :	timer_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Debug autotest",
		 "Command" : 	"debug_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Errno autotest",
		 "Command" : 	"errno_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Meter autotest",
		 "Command" : 	"meter_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Common autotest",
		 "Command" : 	"common_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Dump log history",
		 "Command" :	"dump_log_history",
		 "Func" :	dump_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Dump rings",
		 "Command" :	"dump_ring",
		 "Func" :	dump_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Dump mempools",
		 "Command" :	"dump_mempool",
		 "Func" :	dump_autotest,
		 "Report" :	None,
		},
	]
},
{
	"Prefix":	"group_2",
	"Memory" :	"64",
	"Tests" :
	[
		{
		 "Name" :	"Memory autotest",
		 "Command" :	"memory_autotest",
		 "Func" :	memory_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Read/write lock autotest",
		 "Command" : 	"rwlock_autotest",
		 "Func" :	rwlock_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Logs autotest",
		 "Command" : 	"logs_autotest",
		 "Func" :	logs_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"CPU flags autotest",
		 "Command" : 	"cpuflags_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Version autotest",
		 "Command" : 	"version_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"EAL filesystem autotest",
		 "Command" : 	"eal_fs_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"EAL flags autotest",
		 "Command" : 	"eal_flags_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Hash autotest",
		 "Command" : 	"hash_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
	],
},
{
	"Prefix":	"group_3",
	"Memory" :	per_sockets(1024),
	"Tests" :
	[
		{
		 "Name" :	"LPM autotest",
		 "Command" : 	"lpm_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"IVSHMEM autotest",
		 "Command" : 	"ivshmem_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Memcpy autotest",
		 "Command" : 	"memcpy_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Memzone autotest",
		 "Command" : 	"memzone_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"String autotest",
		 "Command" : 	"string_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Alarm autotest",
		 "Command" :	"alarm_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
	]
},
{
	"Prefix":	"group_4",
	"Memory" :	per_sockets(128),
	"Tests" :
	[
		{
		 "Name" :	"PCI autotest",
		 "Command" :	"pci_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Malloc autotest",
		 "Command" : 	"malloc_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Multi-process autotest",
		 "Command" : 	"multiprocess_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Mbuf autotest",
		 "Command" : 	"mbuf_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Per-lcore autotest",
		 "Command" : 	"per_lcore_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Ring autotest",
		 "Command" : 	"ring_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
	]
},
{
	"Prefix":	"group_5",
	"Memory" :	"32",
	"Tests" :
	[
		{
		 "Name" :	"Spinlock autotest",
		 "Command" : 	"spinlock_autotest",
		 "Func" :	spinlock_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Byte order autotest",
		 "Command" : 	"byteorder_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"TAILQ autotest",
		 "Command" : 	"tailq_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Command-line autotest",
		 "Command" : 	"cmdline_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Interrupts autotest",
		 "Command" : 	"interrupt_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
	]
},
{
	"Prefix":	"group_6",
	"Memory" :	per_sockets(620),
	"Tests" :
	[
		{
		 "Name" :	"Function reentrancy autotest",
		 "Command" : 	"func_reentrancy_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Mempool autotest",
		 "Command" : 	"mempool_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Atomics autotest",
		 "Command" : 	"atomic_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Prefetch autotest",
		 "Command" : 	"prefetch_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		 },
		{
		 "Name" :"Red autotest",
		 "Command" : "red_autotest",
		 "Func" :default_autotest,
		 "Report" :None,
		 },
	]
},
{
	"Prefix" :	"group_7",
	"Memory" :	"400",
	"Tests" :
	[
		{
		 "Name" :	"PMD ring autotest",
		 "Command" :	"ring_pmd_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Access list control autotest",
		 "Command" : 	"acl_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		 {
		 "Name" :"Sched autotest",
		 "Command" : "sched_autotest",
		 "Func" :default_autotest,
		 "Report" :None,
		 },
	]
},
]

# tests that should not be run when any other tests are running
non_parallel_test_group_list = [

{
	"Prefix" :	"kni",
	"Memory" :	"512",
	"Tests" :
	[
		{
		 "Name" :	"KNI autotest",
		 "Command" :	"kni_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
	]
},
{
	"Prefix":	"mempool_perf",
	"Memory" :	per_sockets(256),
	"Tests" :
	[
		{
		 "Name" :	"Cycles autotest",
		 "Command" : 	"cycles_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
		{
		 "Name" :	"Mempool performance autotest",
		 "Command" : 	"mempool_perf_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
	]
},
{
	"Prefix":	"memcpy_perf",
	"Memory" :	per_sockets(512),
	"Tests" :
	[
		{
		 "Name" :	"Memcpy performance autotest",
		 "Command" : 	"memcpy_perf_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
	]
},
{
	"Prefix":	"hash_perf",
	"Memory" :	per_sockets(512),
	"Tests" :
	[
		{
		 "Name" :	"Hash performance autotest",
		 "Command" : 	"hash_perf_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
	]
},
{
	"Prefix" :      "power",
	"Memory" :      per_sockets(512),
	"Tests" :
	[
		{
		 "Name" :       "Power autotest",
		 "Command" :    "power_autotest",
		 "Func" :       default_autotest,
		 "Report" :     None,
		},
	]
},
{
	"Prefix" :      "power_acpi_cpufreq",
	"Memory" :      per_sockets(512),
	"Tests" :
	[
		{
		 "Name" :       "Power ACPI cpufreq autotest",
		 "Command" :    "power_acpi_cpufreq_autotest",
		 "Func" :       default_autotest,
		 "Report" :     None,
		},
	]
},
{
	"Prefix" :      "power_kvm_vm",
	"Memory" :      "512",
	"Tests" :
	[
		{
		 "Name" :       "Power KVM VM  autotest",
		 "Command" :    "power_kvm_vm_autotest",
		 "Func" :       default_autotest,
		 "Report" :     None,
		},
	]
},
{
	"Prefix" :	"lpm6",
	"Memory" :	"512",
	"Tests" :
	[
		{
                 "Name" :       "LPM6 autotest",
                 "Command" :    "lpm6_autotest",
                 "Func" :       default_autotest,
                 "Report" :     None,
                },
	]
},
{
	"Prefix":	"timer_perf",
	"Memory" :	per_sockets(512),
	"Tests" :
	[
		{
		 "Name" :	"Timer performance autotest",
		 "Command" : 	"timer_perf_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
	]
},

#
# Please always make sure that ring_perf is the last test!
#
{
	"Prefix":	"ring_perf",
	"Memory" :	per_sockets(512),
	"Tests" :
	[
		{
		 "Name" :	"Ring performance autotest",
		 "Command" : 	"ring_perf_autotest",
		 "Func" :	default_autotest,
		 "Report" :	None,
		},
	]
},
]



#!/usr/bin/python
#/*
# *   BSD LICENSE
# *
# *   Copyright(c) 2010-2014 Intel Corporation. All rights reserved.
# *   All rights reserved.
# *
# *   Redistribution and use in source and binary forms, with or without
# *   modification, are permitted provided that the following conditions
# *   are met:
# *
# *     * Redistributions of source code must retain the above copyright
# *       notice, this list of conditions and the following disclaimer.
# *     * Redistributions in binary form must reproduce the above copyright
# *       notice, this list of conditions and the following disclaimer in
# *       the documentation and/or other materials provided with the
# *       distribution.
# *     * Neither the name of Intel Corporation nor the names of its
# *       contributors may be used to endorse or promote products derived
# *       from this software without specific prior written permission.
# *
# *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# */

#####################################################################
# This script is designed to modify the call to the QEMU emulator
# to support userspace vhost when starting a guest machine through
# libvirt with vhost enabled. The steps to enable this are as follows
# and should be run as root:
#
# 1. Place this script in a libvirtd's binary search PATH ($PATH)
#    A good location would be in the same directory that the QEMU
#    binary is located
#
# 2. Ensure that the script has the same owner/group and file
#    permissions as the QEMU binary
#
# 3. Update the VM xml file using "virsh edit VM.xml"
#
#    3.a) Set the VM to use the launch script
#
#	Set the emulator path contained in the
#		<emulator><emulator/> tags
#
#	e.g replace <emulator>/usr/bin/qemu-kvm<emulator/>
#        with    <emulator>/usr/bin/qemu-wrap.py<emulator/>
#
#	 3.b) Set the VM's device's to use vhost-net offload
#
#		<interface type="network">
#	<model type="virtio"/>
#	<driver name="vhost"/>
#		<interface/>
#
# 4. Enable libvirt to access our userpace device file by adding it to
#    controllers cgroup for libvirtd using the following steps
#
#   4.a) In /etc/libvirt/qemu.conf add/edit the following lines:
#         1) cgroup_controllers = [ ... "devices", ... ]
#		  2) clear_emulator_capabilities = 0
#         3) user = "root"
#         4) group = "root"
#         5) cgroup_device_acl = [
#                "/dev/null", "/dev/full", "/dev/zero",
#                "/dev/random", "/dev/urandom",
#                "/dev/ptmx", "/dev/kvm", "/dev/kqemu",
#                "/dev/rtc", "/dev/hpet", "/dev/net/tun",
#                "/dev/<devbase-name>",
#                "/dev/hugepages",
#            ]
#
#   4.b) Disable SELinux or set to permissive mode
#
#   4.c) Mount cgroup device controller
#        "mkdir /dev/cgroup"
#        "mount -t cgroup none /dev/cgroup -o devices"
#
#   4.d) Set hugetlbfs_mount variable - ( Optional )
#        VMs using userspace vhost must use hugepage backed
#        memory. This can be enabled in the libvirt XML
#        config by adding a memory backing section to the
#        XML config e.g.
#             <memoryBacking>
#             <hugepages/>
#             </memoryBacking>
#        This memory backing section should be added after the
#        <memory> and <currentMemory> sections. This will add
#        flags "-mem-prealloc -mem-path <path>" to the QEMU
#        command line. The hugetlbfs_mount variable can be used
#        to override the default <path> passed through by libvirt.
#
#        if "-mem-prealloc" or "-mem-path <path>" are not passed
#        through and a vhost device is detected then these options will
#        be automatically added by this script. This script will detect
#        the system hugetlbfs mount point to be used for <path>. The
#        default <path> for this script can be overidden by the
#        hugetlbfs_dir variable in the configuration section of this script.
#
#
#   4.e) Restart the libvirtd system process
#        e.g. on Fedora "systemctl restart libvirtd.service"
#
#
#   4.f) Edit the Configuration Parameters section of this script
#        to point to the correct emulator location and set any
#        addition options
#
# The script modifies the libvirtd Qemu call by modifying/adding
# options based on the configuration parameters below.
# NOTE:
#     emul_path and us_vhost_path must be set
#     All other parameters are optional
#####################################################################


#############################################
# Configuration Parameters
#############################################
#Path to QEMU binary
emul_path = "/usr/local/bin/qemu-system-x86_64"

#Path to userspace vhost device file
# This filename should match the --dev-basename parameters of
# the command used to launch the userspace vhost sample application e.g.
# if the sample app lauch command is:
#    ./build/vhost-switch ..... --dev-basename usvhost
# then this variable should be set to:
#   us_vhost_path = "/dev/usvhost"
us_vhost_path = "/dev/usvhost"

#List of additional user defined emulation options. These options will
#be added to all Qemu calls
emul_opts_user = []

#List of additional user defined emulation options for vhost only.
#These options will only be added to vhost enabled guests
emul_opts_user_vhost = []

#For all VHOST enabled VMs, the VM memory is preallocated from hugetlbfs
# Set this variable to one to enable this option for all VMs
use_huge_all = 0

#Instead of autodetecting, override the hugetlbfs directory by setting
#this variable
hugetlbfs_dir = ""

#############################################


#############################################
# ****** Do Not Modify Below this Line ******
#############################################

import sys, os, subprocess
import time
import signal


#List of open userspace vhost file descriptors
fd_list = []

#additional virtio device flags when using userspace vhost
vhost_flags = [ "csum=off",
                "gso=off",
                "guest_tso4=off",
                "guest_tso6=off",
                "guest_ecn=off"
              ]

#String of the path to the Qemu process pid
qemu_pid = "/tmp/%d-qemu.pid" % os.getpid()

#############################################
# Signal haldler to kill Qemu subprocess
#############################################
def kill_qemu_process(signum, stack):
    pidfile = open(qemu_pid, 'r')
    pid = int(pidfile.read())
    os.killpg(pid, signal.SIGTERM)
    pidfile.close()


#############################################
# Find the system hugefile mount point.
# Note:
# if multiple hugetlbfs mount points exist
# then the first one found will be used
#############################################
def find_huge_mount():

    if (len(hugetlbfs_dir)):
        return hugetlbfs_dir

    huge_mount = ""

    if (os.access("/proc/mounts", os.F_OK)):
        f = open("/proc/mounts", "r")
        line = f.readline()
        while line:
            line_split = line.split(" ")
            if line_split[2] == 'hugetlbfs':
                huge_mount = line_split[1]
                break
            line = f.readline()
    else:
        print "/proc/mounts not found"
        exit (1)

    f.close
    if len(huge_mount) == 0:
        print "Failed to find hugetlbfs mount point"
        exit (1)

    return huge_mount


#############################################
# Get a userspace Vhost file descriptor
#############################################
def get_vhost_fd():

    if (os.access(us_vhost_path, os.F_OK)):
        fd = os.open( us_vhost_path, os.O_RDWR)
    else:
        print ("US-Vhost file %s not found" %us_vhost_path)
        exit (1)

    return fd


#############################################
# Check for vhostfd. if found then replace
# with our own vhost fd and append any vhost
# flags onto the end
#############################################
def modify_netdev_arg(arg):

    global fd_list
    vhost_in_use = 0
    s = ''
    new_opts = []
    netdev_opts = arg.split(",")

    for opt in netdev_opts:
        #check if vhost is used
        if "vhost" == opt[:5]:
            vhost_in_use = 1
        else:
            new_opts.append(opt)

    #if using vhost append vhost options
    if vhost_in_use == 1:
        #append vhost on option
        new_opts.append('vhost=on')
        #append vhostfd ption
        new_fd = get_vhost_fd()
        new_opts.append('vhostfd=' + str(new_fd))
        fd_list.append(new_fd)

    #concatenate all options
    for opt in new_opts:
        if len(s) > 0:
			s+=','

        s+=opt

    return s


#############################################
# Main
#############################################
def main():

    global fd_list
    global vhost_in_use
    new_args = []
    num_cmd_args = len(sys.argv)
    emul_call = ''
    mem_prealloc_set = 0
    mem_path_set = 0
    num = 0;

    #parse the parameters
    while (num < num_cmd_args):
        arg = sys.argv[num]

	#Check netdev +1 parameter for vhostfd
        if arg == '-netdev':
            num_vhost_devs = len(fd_list)
            new_args.append(arg)

            num+=1
            arg = sys.argv[num]
            mod_arg = modify_netdev_arg(arg)
            new_args.append(mod_arg)

            #append vhost flags if this is a vhost device
            # and -device is the next arg
            # i.e -device -opt1,-opt2,...,-opt3,%vhost
            if (num_vhost_devs < len(fd_list)):
                num+=1
                arg = sys.argv[num]
                if arg == '-device':
                    new_args.append(arg)
                    num+=1
                    new_arg = sys.argv[num]
                    for flag in vhost_flags:
                        new_arg = ''.join([new_arg,',',flag])
                    new_args.append(new_arg)
                else:
                    new_args.append(arg)
        elif arg == '-mem-prealloc':
            mem_prealloc_set = 1
            new_args.append(arg)
        elif arg == '-mem-path':
            mem_path_set = 1
            new_args.append(arg)

        else:
            new_args.append(arg)

        num+=1

    #Set Qemu binary location
    emul_call+=emul_path
    emul_call+=" "

    #Add prealloc mem options if using vhost and not already added
    if ((len(fd_list) > 0) and (mem_prealloc_set == 0)):
        emul_call += "-mem-prealloc "

    #Add mempath mem options if using vhost and not already added
    if ((len(fd_list) > 0) and (mem_path_set == 0)):
        #Detect and add hugetlbfs mount point
        mp = find_huge_mount()
        mp = "".join(["-mem-path ", mp])
        emul_call += mp
        emul_call += " "

    #add user options
    for opt in emul_opts_user:
        emul_call += opt
        emul_call += " "

    #Add add user vhost only options
    if len(fd_list) > 0:
        for opt in emul_opts_user_vhost:
            emul_call += opt
            emul_call += " "

    #Add updated libvirt options
    iter_args = iter(new_args)
    #skip 1st arg i.e. call to this script
    next(iter_args)
    for arg in iter_args:
        emul_call+=str(arg)
        emul_call+= " "

    emul_call += "-pidfile %s " % qemu_pid
    #Call QEMU
    process = subprocess.Popen(emul_call, shell=True, preexec_fn=os.setsid)

    for sig in [signal.SIGTERM, signal.SIGINT, signal.SIGHUP, signal.SIGQUIT]:
        signal.signal(sig, kill_qemu_process)

    process.wait()

    #Close usvhost files
    for fd in fd_list:
        os.close(fd)
    #Cleanup temporary files
    if os.access(qemu_pid, os.F_OK):
        os.remove(qemu_pid)

if __name__ == "__main__":
    main()



#! /usr/bin/python
#
#   BSD LICENSE
#
#   Copyright(c) 2010-2014 Intel Corporation. All rights reserved.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     * Neither the name of Intel Corporation nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import sys, os, getopt, subprocess
from os.path import exists, abspath, dirname, basename


# The PCI device class for ETHERNET devices
ETHERNET_CLASS = "0200"

# global dict ethernet devices present. Dictionary indexed by PCI address.
# Each device within this is itself a dictionary of device properties
devices = {}
# list of supported DPDK drivers
dpdk_drivers = [ "igb_uio", "vfio-pci", "uio_pci_generic" ]

# command-line arg flags
b_flag = None
status_flag = False
force_flag = False
args = []

def usage():
    '''Print usage information for the program'''
    argv0 = basename(sys.argv[0])
    print """
Usage:
------

     %(argv0)s [options] DEVICE1 DEVICE2 ....

where DEVICE1, DEVICE2 etc, are specified via PCI "domain:bus:slot.func" syntax
or "bus:slot.func" syntax. For devices bound to Linux kernel drivers, they may
also be referred to by Linux interface name e.g. eth0, eth1, em0, em1, etc.

Options:
    --help, --usage:
        Display usage information and quit

    -s, --status:
        Print the current status of all known network interfaces.
        For each device, it displays the PCI domain, bus, slot and function,
        along with a text description of the device. Depending upon whether the
        device is being used by a kernel driver, the igb_uio driver, or no
        driver, other relevant information will be displayed:
        * the Linux interface name e.g. if=eth0
        * the driver being used e.g. drv=igb_uio
        * any suitable drivers not currently using that device
            e.g. unused=igb_uio
        NOTE: if this flag is passed along with a bind/unbind option, the status
        display will always occur after the other operations have taken place.

    -b driver, --bind=driver:
        Select the driver to use or \"none\" to unbind the device

    -u, --unbind:
        Unbind a device (Equivalent to \"-b none\")

    --force:
        By default, devices which are used by Linux - as indicated by having
        routes in the routing table - cannot be modified. Using the --force
        flag overrides this behavior, allowing active links to be forcibly
        unbound.
        WARNING: This can lead to loss of network connection and should be used
        with caution.

Examples:
---------

To display current device status:
        %(argv0)s --status

To bind eth1 from the current driver and move to use igb_uio
        %(argv0)s --bind=igb_uio eth1

To unbind 0000:01:00.0 from using any driver
        %(argv0)s -u 0000:01:00.0

To bind 0000:02:00.0 and 0000:02:00.1 to the ixgbe kernel driver
        %(argv0)s -b ixgbe 02:00.0 02:00.1

    """ % locals() # replace items from local variables

# This is roughly compatible with check_output function in subprocess module
# which is only available in python 2.7.
def check_output(args, stderr=None):
    '''Run a command and capture its output'''
    return subprocess.Popen(args, stdout=subprocess.PIPE,
                            stderr=stderr).communicate()[0]

def find_module(mod):
    '''find the .ko file for kernel module named mod.
    Searches the $RTE_SDK/$RTE_TARGET directory, the kernel
    modules directory and finally under the parent directory of
    the script '''
    # check $RTE_SDK/$RTE_TARGET directory
    if 'RTE_SDK' in os.environ and 'RTE_TARGET' in os.environ:
        path = "%s/%s/kmod/%s.ko" % (os.environ['RTE_SDK'],\
                                     os.environ['RTE_TARGET'], mod)
        if exists(path):
            return path

    # check using depmod
    try:
        depmod_out = check_output(["modinfo", "-n", mod], \
                                  stderr=subprocess.STDOUT).lower()
        if "error" not in depmod_out:
            path = depmod_out.strip()
            if exists(path):
                return path
    except: # if modinfo can't find module, it fails, so continue
        pass

    # check for a copy based off current path
    tools_dir = dirname(abspath(sys.argv[0]))
    if (tools_dir.endswith("tools")):
        base_dir = dirname(tools_dir)
        find_out = check_output(["find", base_dir, "-name", mod + ".ko"])
        if len(find_out) > 0: #something matched
            path = find_out.splitlines()[0]
            if exists(path):
                return path

def check_modules():
    '''Checks that igb_uio is loaded'''
    global dpdk_drivers

    fd = file("/proc/modules")
    loaded_mods = fd.readlines()
    fd.close()

    # list of supported modules
    mods =  [{"Name" : driver, "Found" : False} for driver in dpdk_drivers]

    # first check if module is loaded
    for line in loaded_mods:
        for mod in mods:
            if line.startswith(mod["Name"]):
                mod["Found"] = True
            # special case for vfio_pci (module is named vfio-pci,
            # but its .ko is named vfio_pci)
            elif line.replace("_", "-").startswith(mod["Name"]):
                mod["Found"] = True

    # check if we have at least one loaded module
    if True not in [mod["Found"] for mod in mods] and b_flag is not None:
        if b_flag in dpdk_drivers:
            print "Error - no supported modules(DPDK driver) are loaded"
            sys.exit(1)
        else:
            print "Warning - no supported modules(DPDK driver) are loaded"

    # change DPDK driver list to only contain drivers that are loaded
    dpdk_drivers = [mod["Name"] for mod in mods if mod["Found"]]

def has_driver(dev_id):
    '''return true if a device is assigned to a driver. False otherwise'''
    return "Driver_str" in devices[dev_id]

def get_pci_device_details(dev_id):
    '''This function gets additional details for a PCI device'''
    device = {}

    extra_info = check_output(["lspci", "-vmmks", dev_id]).splitlines()

    # parse lspci details
    for line in extra_info:
        if len(line) == 0:
            continue
        name, value = line.split("\t", 1)
        name = name.strip(":") + "_str"
        device[name] = value
    # check for a unix interface name
    sys_path = "/sys/bus/pci/devices/%s/net/" % dev_id
    if exists(sys_path):
        device["Interface"] = ",".join(os.listdir(sys_path))
    else:
        device["Interface"] = ""
    # check if a port is used for ssh connection
    device["Ssh_if"] = False
    device["Active"] = ""

    return device

def get_nic_details():
    '''This function populates the "devices" dictionary. The keys used are
    the pci addresses (domain:bus:slot.func). The values are themselves
    dictionaries - one for each NIC.'''
    global devices
    global dpdk_drivers

    # clear any old data
    devices = {}
    # first loop through and read details for all devices
    # request machine readable format, with numeric IDs
    dev = {};
    dev_lines = check_output(["lspci", "-Dvmmn"]).splitlines()
    for dev_line in dev_lines:
        if (len(dev_line) == 0):
            if dev["Class"] == ETHERNET_CLASS:
                #convert device and vendor ids to numbers, then add to global
                dev["Vendor"] = int(dev["Vendor"],16)
                dev["Device"] = int(dev["Device"],16)
                devices[dev["Slot"]] = dict(dev) # use dict to make copy of dev
        else:
            name, value = dev_line.split("\t", 1)
            dev[name.rstrip(":")] = value

    # check what is the interface if any for an ssh connection if
    # any to this host, so we can mark it later.
    ssh_if = []
    route = check_output(["ip", "-o", "route"])
    # filter out all lines for 169.254 routes
    route = "\n".join(filter(lambda ln: not ln.startswith("169.254"),
                             route.splitlines()))
    rt_info = route.split()
    for i in xrange(len(rt_info) - 1):
        if rt_info[i] == "dev":
            ssh_if.append(rt_info[i+1])

    # based on the basic info, get extended text details
    for d in devices.keys():
        # get additional info and add it to existing data
        devices[d] = dict(devices[d].items() +
                          get_pci_device_details(d).items())

        for _if in ssh_if:
            if _if in devices[d]["Interface"].split(","):
                devices[d]["Ssh_if"] = True
                devices[d]["Active"] = "*Active*"
                break;

        # add igb_uio to list of supporting modules if needed
        if "Module_str" in devices[d]:
            for driver in dpdk_drivers:
                if driver not in devices[d]["Module_str"]:
                    devices[d]["Module_str"] = devices[d]["Module_str"] + ",%s" % driver
        else:
            devices[d]["Module_str"] = ",".join(dpdk_drivers)

        # make sure the driver and module strings do not have any duplicates
        if has_driver(d):
            modules = devices[d]["Module_str"].split(",")
            if devices[d]["Driver_str"] in modules:
                modules.remove(devices[d]["Driver_str"])
                devices[d]["Module_str"] = ",".join(modules)

def dev_id_from_dev_name(dev_name):
    '''Take a device "name" - a string passed in by user to identify a NIC
    device, and determine the device id - i.e. the domain:bus:slot.func - for
    it, which can then be used to index into the devices array'''
    dev = None
    # check if it's already a suitable index
    if dev_name in devices:
        return dev_name
    # check if it's an index just missing the domain part
    elif "0000:" + dev_name in devices:
        return "0000:" + dev_name
    else:
        # check if it's an interface name, e.g. eth1
        for d in devices.keys():
            if dev_name in devices[d]["Interface"].split(","):
                return devices[d]["Slot"]
    # if nothing else matches - error
    print "Unknown device: %s. " \
        "Please specify device in \"bus:slot.func\" format" % dev_name
    sys.exit(1)

def unbind_one(dev_id, force):
    '''Unbind the device identified by "dev_id" from its current driver'''
    dev = devices[dev_id]
    if not has_driver(dev_id):
        print "%s %s %s is not currently managed by any driver\n" % \
            (dev["Slot"], dev["Device_str"], dev["Interface"])
        return

    # prevent us disconnecting ourselves
    if dev["Ssh_if"] and not force:
        print "Routing table indicates that interface %s is active" \
            ". Skipping unbind" % (dev_id)
        return

    # write to /sys to unbind
    filename = "/sys/bus/pci/drivers/%s/unbind" % dev["Driver_str"]
    try:
        f = open(filename, "a")
    except:
        print "Error: unbind failed for %s - Cannot open %s" % (dev_id, filename)
        sys/exit(1)
    f.write(dev_id)
    f.close()

def bind_one(dev_id, driver, force):
    '''Bind the device given by "dev_id" to the driver "driver". If the device
    is already bound to a different driver, it will be unbound first'''
    dev = devices[dev_id]
    saved_driver = None # used to rollback any unbind in case of failure

    # prevent disconnection of our ssh session
    if dev["Ssh_if"] and not force:
        print "Routing table indicates that interface %s is active" \
            ". Not modifying" % (dev_id)
        return

    # unbind any existing drivers we don't want
    if has_driver(dev_id):
        if dev["Driver_str"] == driver:
            print "%s already bound to driver %s, skipping\n" % (dev_id, driver)
            return
        else:
            saved_driver = dev["Driver_str"]
            unbind_one(dev_id, force)
            dev["Driver_str"] = "" # clear driver string

    # if we are binding to one of DPDK drivers, add PCI id's to that driver
    if driver in dpdk_drivers:
        filename = "/sys/bus/pci/drivers/%s/new_id" % driver
        try:
            f = open(filename, "w")
        except:
            print "Error: bind failed for %s - Cannot open %s" % (dev_id, filename)
            return
        try:
            f.write("%04x %04x" % (dev["Vendor"], dev["Device"]))
            f.close()
        except:
            print "Error: bind failed for %s - Cannot write new PCI ID to " \
                "driver %s" % (dev_id, driver)
            return

    # do the bind by writing to /sys
    filename = "/sys/bus/pci/drivers/%s/bind" % driver
    try:
        f = open(filename, "a")
    except:
        print "Error: bind failed for %s - Cannot open %s" % (dev_id, filename)
        if saved_driver is not None: # restore any previous driver
            bind_one(dev_id, saved_driver, force)
        return
    try:
        f.write(dev_id)
        f.close()
    except:
        # for some reason, closing dev_id after adding a new PCI ID to new_id
        # results in IOError. however, if the device was successfully bound,
        # we don't care for any errors and can safely ignore IOError
        tmp = get_pci_device_details(dev_id)
        if "Driver_str" in tmp and tmp["Driver_str"] == driver:
            return
        print "Error: bind failed for %s - Cannot bind to driver %s" % (dev_id, driver)
        if saved_driver is not None: # restore any previous driver
            bind_one(dev_id, saved_driver, force)
        return


def unbind_all(dev_list, force=False):
    """Unbind method, takes a list of device locations"""
    dev_list = map(dev_id_from_dev_name, dev_list)
    for d in dev_list:
        unbind_one(d, force)

def bind_all(dev_list, driver, force=False):
    """Bind method, takes a list of device locations"""
    global devices

    dev_list = map(dev_id_from_dev_name, dev_list)

    for d in dev_list:
        bind_one(d, driver, force)

    # when binding devices to a generic driver (i.e. one that doesn't have a
    # PCI ID table), some devices that are not bound to any other driver could
    # be bound even if no one has asked them to. hence, we check the list of
    # drivers again, and see if some of the previously-unbound devices were
    # erroneously bound.
    for d in devices.keys():
        # skip devices that were already bound or that we know should be bound
        if "Driver_str" in devices[d] or d in dev_list:
            continue

        # update information about this device
        devices[d] = dict(devices[d].items() +
                          get_pci_device_details(d).items())

        # check if updated information indicates that the device was bound
        if "Driver_str" in devices[d]:
            unbind_one(d, force)

def display_devices(title, dev_list, extra_params = None):
    '''Displays to the user the details of a list of devices given in "dev_list"
    The "extra_params" parameter, if given, should contain a string with
    %()s fields in it for replacement by the named fields in each device's
    dictionary.'''
    strings = [] # this holds the strings to print. We sort before printing
    print "\n%s" % title
    print   "="*len(title)
    if len(dev_list) == 0:
        strings.append("<none>")
    else:
        for dev in dev_list:
            if extra_params is not None:
                strings.append("%s '%s' %s" % (dev["Slot"], \
                                dev["Device_str"], extra_params % dev))
            else:
                strings.append("%s '%s'" % (dev["Slot"], dev["Device_str"]))
    # sort before printing, so that the entries appear in PCI order
    strings.sort()
    print "\n".join(strings) # print one per line

def show_status():
    '''Function called when the script is passed the "--status" option. Displays
    to the user what devices are bound to the igb_uio driver, the kernel driver
    or to no driver'''
    global dpdk_drivers
    kernel_drv = []
    dpdk_drv = []
    no_drv = []

    # split our list of devices into the three categories above
    for d in devices.keys():
        if not has_driver(d):
            no_drv.append(devices[d])
            continue
        if devices[d]["Driver_str"] in dpdk_drivers:
            dpdk_drv.append(devices[d])
        else:
            kernel_drv.append(devices[d])

    # print each category separately, so we can clearly see what's used by DPDK
    display_devices("Network devices using DPDK-compatible driver", dpdk_drv, \
                    "drv=%(Driver_str)s unused=%(Module_str)s")
    display_devices("Network devices using kernel driver", kernel_drv,
                    "if=%(Interface)s drv=%(Driver_str)s unused=%(Module_str)s %(Active)s")
    display_devices("Other network devices", no_drv,\
                    "unused=%(Module_str)s")

def parse_args():
    '''Parses the command-line arguments given by the user and takes the
    appropriate action for each'''
    global b_flag
    global status_flag
    global force_flag
    global args
    if len(sys.argv) <= 1:
        usage()
        sys.exit(0)

    try:
        opts, args = getopt.getopt(sys.argv[1:], "b:us",
                               ["help", "usage", "status", "force",
                                "bind=", "unbind"])
    except getopt.GetoptError, error:
        print str(error)
        print "Run '%s --usage' for further information" % sys.argv[0]
        sys.exit(1)

    for opt, arg in opts:
        if opt == "--help" or opt == "--usage":
            usage()
            sys.exit(0)
        if opt == "--status" or opt == "-s":
            status_flag = True
        if opt == "--force":
            force_flag = True
        if opt == "-b" or opt == "-u" or opt == "--bind" or opt == "--unbind":
            if b_flag is not None:
                print "Error - Only one bind or unbind may be specified\n"
                sys.exit(1)
            if opt == "-u" or opt == "--unbind":
                b_flag = "none"
            else:
                b_flag = arg

def do_arg_actions():
    '''do the actual action requested by the user'''
    global b_flag
    global status_flag
    global force_flag
    global args

    if b_flag is None and not status_flag:
        print "Error: No action specified for devices. Please give a -b or -u option"
        print "Run '%s --usage' for further information" % sys.argv[0]
        sys.exit(1)

    if b_flag is not None and len(args) == 0:
        print "Error: No devices specified."
        print "Run '%s --usage' for further information" % sys.argv[0]
        sys.exit(1)

    if b_flag == "none" or b_flag == "None":
        unbind_all(args, force_flag)
    elif b_flag is not None:
        bind_all(args, b_flag, force_flag)
    if status_flag:
        if b_flag is not None:
            get_nic_details() # refresh if we have changed anything
        show_status()

def main():
    '''program main function'''
    parse_args()
    check_modules()
    get_nic_details()
    do_arg_actions()

if __name__ == "__main__":
    main()



#!/usr/bin/python

#   BSD LICENSE
#
#   Copyright(c) 2010-2014 Intel Corporation. All rights reserved.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     * Neither the name of Intel Corporation nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Script that uses either test app or qemu controlled by python-pexpect

import sys, autotest_data, autotest_runner



def usage():
	print"Usage: autotest.py [test app|test iso image]",
	print "[target] [whitelist|-blacklist]"

if len(sys.argv) < 3:
	usage()
	sys.exit(1)

target = sys.argv[2]

test_whitelist=None
test_blacklist=None

# get blacklist/whitelist
if len(sys.argv) > 3:
	testlist = sys.argv[3].split(',')
	testlist = [test.lower() for test in testlist]
	if testlist[0].startswith('-'):
		testlist[0] = testlist[0].lstrip('-')
		test_blacklist = testlist
	else:
		test_whitelist = testlist

cmdline  = "%s -c f -n 4"%(sys.argv[1])

print cmdline

runner = autotest_runner.AutotestRunner(cmdline, target, test_blacklist, test_whitelist)

for test_group in autotest_data.parallel_test_group_list:
	runner.add_parallel_test_group(test_group)

for test_group in autotest_data.non_parallel_test_group_list:
	runner.add_non_parallel_test_group(test_group)

num_fails = runner.run_all_tests()

sys.exit(num_fails)



#   BSD LICENSE
#   Copyright(c) 2010-2015 Intel Corporation. All rights reserved.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#   * Neither the name of Intel Corporation nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import subprocess
from docutils import nodes
from distutils.version import LooseVersion
from sphinx import __version__ as sphinx_version
from sphinx.highlighting import PygmentsBridge
from pygments.formatters.latex import LatexFormatter

project = 'Data Plane Development Kit'

if LooseVersion(sphinx_version) >= LooseVersion('1.3.1'):
    html_theme = "sphinx_rtd_theme"
html_logo = '../logo/DPDK_logo_vertical_rev_small.png'
latex_logo = '../logo/DPDK_logo_horizontal_tag.png'
html_add_permalinks = ""
html_show_copyright = False
highlight_language = 'none'

version = subprocess.check_output(['make', '-sRrC', '../../', 'showversion']).decode('utf-8')
release = version

master_doc = 'index'

# Figures, tables and code-blocks automatically numbered if they have caption
numfig = True

latex_documents = [
    ('index',
     'doc.tex',
     '',
     '',
     'manual')
]

# Latex directives to be included directly in the latex/pdf docs.
latex_preamble = r"""
\usepackage[utf8]{inputenc}
\usepackage[T1]{fontenc}
\usepackage{helvet}
\renewcommand{\familydefault}{\sfdefault}
\RecustomVerbatimEnvironment{Verbatim}{Verbatim}{xleftmargin=5mm}
"""

# Configuration for the latex/pdf docs.
latex_elements = {
    'papersize': 'a4paper',
    'pointsize': '11pt',
    # remove blank pages
    'classoptions': ',openany,oneside',
    'babel': '\\usepackage[english]{babel}',
    # customize Latex formatting
    'preamble': latex_preamble
}

# Override the default Latex formatter in order to modify the
# code/verbatim blocks.
class CustomLatexFormatter(LatexFormatter):
    def __init__(self, **options):
        super(CustomLatexFormatter, self).__init__(**options)
        # Use the second smallest font size for code/verbatim blocks.
        self.verboptions = r'formatcom=\footnotesize'

# Replace the default latex formatter.
PygmentsBridge.latex_formatter = CustomLatexFormatter

######## :numref: fallback ########
# The following hook functions add some simple handling for the :numref:
# directive for Sphinx versions prior to 1.3.1. The functions replace the
# :numref: reference with a link to the target (for all Sphinx doc types).
# It doesn't try to label figures/tables.

def numref_role(reftype, rawtext, text, lineno, inliner):
    """
    Add a Sphinx role to handle numref references. Note, we can't convert
    the link here because the doctree isn't build and the target information
    isn't available.
    """
    # Add an identifier to distinguish numref from other references.
    newnode = nodes.reference('',
                              '',
                              refuri='_local_numref_#%s' % text,
                              internal=True)
    return [newnode], []

def process_numref(app, doctree, from_docname):
    """
    Process the numref nodes once the doctree has been built and prior to
    writing the files. The processing involves replacing the numref with a
    link plus text to indicate if it is a Figure or Table link.
    """

    # Iterate over the reference nodes in the doctree.
    for node in doctree.traverse(nodes.reference):
        target = node.get('refuri', '')

        # Look for numref nodes.
        if target.startswith('_local_numref_#'):
            target = target.replace('_local_numref_#', '')

            # Get the target label and link information from the Sphinx env.
            data = app.builder.env.domains['std'].data
            docname, label, _ = data['labels'].get(target, ('', '', ''))
            relative_url = app.builder.get_relative_uri(from_docname, docname)

            # Add a text label to the link.
            if target.startswith('figure'):
                caption = 'Figure'
            elif target.startswith('table'):
                caption = 'Table'
            else:
                caption = 'Link'

            # New reference node with the updated link information.
            newnode = nodes.reference('',
                                      caption,
                                      refuri='%s#%s' % (relative_url, label),
                                      internal=True)
            node.replace_self(newnode)

def setup(app):
    if LooseVersion(sphinx_version) < LooseVersion('1.3.1'):
        print('Upgrade sphinx to version >= 1.3.1 for '
              'improved Figure/Table number handling.')
        # Add a role to handle :numref: references.
        app.add_role('numref', numref_role)
        # Process the numref references once the doctree has been created.
        app.connect('doctree-resolved', process_numref)



#!/usr/bin/python

#   BSD LICENSE
#
#   Copyright(c) 2010-2014 Intel Corporation. All rights reserved.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     * Neither the name of Intel Corporation nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Test functions

import sys, pexpect, time, os, re

# default autotest, used to run most tests
# waits for "Test OK"
def default_autotest(child, test_name):
	child.sendline(test_name)
	result = child.expect(["Test OK", "Test Failed",
		"Command not found", pexpect.TIMEOUT], timeout = 900)
	if result == 1:
		return -1, "Fail"
	elif result == 2:
		return -1, "Fail [Not found]"
	elif result == 3:
		return -1, "Fail [Timeout]"
	return 0, "Success"

# autotest used to run dump commands
# just fires the command
def dump_autotest(child, test_name):
	child.sendline(test_name)
	return 0, "Success"

# memory autotest
# reads output and waits for Test OK
def memory_autotest(child, test_name):
	child.sendline(test_name)
	regexp = "phys:0x[0-9a-f]*, len:([0-9]*), virt:0x[0-9a-f]*, socket_id:[0-9]*"
	index = child.expect([regexp, pexpect.TIMEOUT], timeout = 180)
	if index != 0:
		return -1, "Fail [Timeout]"
	size = int(child.match.groups()[0], 16)
	if size <= 0:
		return -1, "Fail [Bad size]"
	index = child.expect(["Test OK", "Test Failed",
		          pexpect.TIMEOUT], timeout = 10)
	if index == 1:
		return -1, "Fail"
	elif index == 2:
		return -1, "Fail [Timeout]"
	return 0, "Success"

def spinlock_autotest(child, test_name):
	i = 0
	ir = 0
	child.sendline(test_name)
	while True:
		index = child.expect(["Test OK",
			"Test Failed",
			"Hello from core ([0-9]*) !",
			"Hello from within recursive locks from ([0-9]*) !",
		pexpect.TIMEOUT], timeout = 20)
		# ok
		if index == 0:
			break

		# message, check ordering
		elif index == 2:
			if int(child.match.groups()[0]) < i:
				return -1, "Fail [Bad order]"
			i = int(child.match.groups()[0])
		elif index == 3:
			if int(child.match.groups()[0]) < ir:
				return -1, "Fail [Bad order]"
			ir = int(child.match.groups()[0])

		# fail
		elif index == 4:
			return -1, "Fail [Timeout]"
		elif index == 1:
			return -1, "Fail"

	return 0, "Success"

def rwlock_autotest(child, test_name):
	i = 0
	child.sendline(test_name)
	while True:
		index = child.expect(["Test OK",
			"Test Failed",
			"Hello from core ([0-9]*) !",
			"Global write lock taken on master core ([0-9]*)",
		pexpect.TIMEOUT], timeout = 10)
		# ok
		if index == 0:
			if i != 0xffff:
				return -1, "Fail [Message is missing]"
			break

		# message, check ordering
		elif index == 2:
			if int(child.match.groups()[0]) < i:
				return -1, "Fail [Bad order]"
			i = int(child.match.groups()[0])

		# must be the last message, check ordering
		elif index == 3:
			i = 0xffff

		elif index == 4:
			return -1, "Fail [Timeout]"

		# fail
		else:
			return -1, "Fail"

	return 0, "Success"

def logs_autotest(child, test_name):
	i = 0
	child.sendline(test_name)

	log_list = [
		"TESTAPP1: this is a debug level message",
		"TESTAPP1: this is a info level message",
		"TESTAPP1: this is a warning level message",
		"TESTAPP2: this is a info level message",
		"TESTAPP2: this is a warning level message",
		"TESTAPP1: this is a debug level message",
		"TESTAPP1: this is a debug level message",
		"TESTAPP1: this is a info level message",
		"TESTAPP1: this is a warning level message",
		"TESTAPP2: this is a info level message",
		"TESTAPP2: this is a warning level message",
		"TESTAPP1: this is a debug level message",
	]

	for log_msg in log_list:
		index = child.expect([log_msg,
				      "Test OK",
				      "Test Failed",
				      pexpect.TIMEOUT], timeout = 10)

		if index == 3:
			return -1, "Fail [Timeout]"
		# not ok
		elif index != 0:
			return -1, "Fail"

	index = child.expect(["Test OK",
		"Test Failed",
		pexpect.TIMEOUT], timeout = 10)

	return 0, "Success"

def timer_autotest(child, test_name):
	i = 0
	child.sendline(test_name)

	index = child.expect(["Start timer stress tests \(20 seconds\)",
		"Test Failed",
		pexpect.TIMEOUT], timeout = 10)

	if index == 1:
		return -1, "Fail"
	elif index == 2:
		return -1, "Fail [Timeout]"

	index = child.expect(["Start timer stress tests 2",
		"Test Failed",
		pexpect.TIMEOUT], timeout = 40)

	if index == 1:
		return -1, "Fail"
	elif index == 2:
		return -1, "Fail [Timeout]"

	index = child.expect(["Start timer basic tests \(20 seconds\)",
		"Test Failed",
		pexpect.TIMEOUT], timeout = 20)

	if index == 1:
		return -1, "Fail"
	elif index == 2:
		return -1, "Fail [Timeout]"

	prev_lcore_timer1 = -1

	lcore_tim0 = -1
	lcore_tim1 = -1
	lcore_tim2 = -1
	lcore_tim3 = -1

	while True:
		index = child.expect(["TESTTIMER: ([0-9]*): callback id=([0-9]*) count=([0-9]*) on core ([0-9]*)",
			"Test OK",
			"Test Failed",
			pexpect.TIMEOUT], timeout = 10)

		if index == 1:
			break

		if index == 2:
			return -1, "Fail"
		elif index == 3:
			return -1, "Fail [Timeout]"

		try:
			t = int(child.match.groups()[0])
			id = int(child.match.groups()[1])
			cnt = int(child.match.groups()[2])
			lcore = int(child.match.groups()[3])
		except:
			return -1, "Fail [Cannot parse]"

		# timer0 always expires on the same core when cnt < 20
		if id == 0:
			if lcore_tim0 == -1:
				lcore_tim0 = lcore
			elif lcore != lcore_tim0 and cnt < 20:
				return -1, "Fail [lcore != lcore_tim0 (%d, %d)]"%(lcore, lcore_tim0)
			if cnt > 21:
				return -1, "Fail [tim0 cnt > 21]"

		# timer1 each time expires on a different core
		if id == 1:
			if lcore == lcore_tim1:
				return -1, "Fail [lcore == lcore_tim1 (%d, %d)]"%(lcore, lcore_tim1)
			lcore_tim1 = lcore
			if cnt > 10:
				return -1, "Fail [tim1 cnt > 30]"

		# timer0 always expires on the same core
		if id == 2:
			if lcore_tim2 == -1:
				lcore_tim2 = lcore
			elif lcore != lcore_tim2:
				return -1, "Fail [lcore != lcore_tim2 (%d, %d)]"%(lcore, lcore_tim2)
			if cnt > 30:
				return -1, "Fail [tim2 cnt > 30]"

		# timer0 always expires on the same core
		if id == 3:
			if lcore_tim3 == -1:
				lcore_tim3 = lcore
			elif lcore != lcore_tim3:
				return -1, "Fail [lcore_tim3 changed (%d -> %d)]"%(lcore, lcore_tim3)
			if cnt > 30:
				return -1, "Fail [tim3 cnt > 30]"

	# must be 2 different cores
	if lcore_tim0 == lcore_tim3:
		return -1, "Fail [lcore_tim0 (%d) == lcore_tim3 (%d)]"%(lcore_tim0, lcore_tim3)

	return 0, "Success"

def ring_autotest(child, test_name):
	child.sendline(test_name)
	index = child.expect(["Test OK", "Test Failed",
		pexpect.TIMEOUT], timeout = 15)
	if index == 1:
		return -1, "Fail"
	elif index == 2:
		return -1, "Fail [Timeout]"

	child.sendline("set_watermark test 100")
	child.sendline("dump_ring test")
	index = child.expect(["  watermark=100",
		pexpect.TIMEOUT], timeout = 1)
	if index != 0:
		return -1, "Fail [Bad watermark]"

	return 0, "Success"



#!/usr/bin/python

#   BSD LICENSE
#
#   Copyright(c) 2010-2014 Intel Corporation. All rights reserved.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     * Neither the name of Intel Corporation nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# collection of static data

import sys

# keycode constants
CTRL_A = chr(1)
CTRL_B = chr(2)
CTRL_C = chr(3)
CTRL_D = chr(4)
CTRL_E = chr(5)
CTRL_F = chr(6)
CTRL_K = chr(11)
CTRL_L = chr(12)
CTRL_N = chr(14)
CTRL_P = chr(16)
CTRL_W = chr(23)
CTRL_Y = chr(25)
ALT_B = chr(27) + chr(98)
ALT_D = chr(27) + chr(100)
ALT_F = chr(27) + chr(102)
ALT_BKSPACE = chr(27) + chr(127)
DEL = chr(27) + chr(91) + chr(51) + chr(126)
TAB = chr(9)
HELP = chr(63)
BKSPACE = chr(127)
RIGHT = chr(27) + chr(91) + chr(67)
DOWN = chr(27) + chr(91) + chr(66)
LEFT = chr(27) + chr(91) + chr(68)
UP = chr(27) + chr(91) + chr(65)
ENTER2 = '\r'
ENTER = '\n'

# expected result constants
NOT_FOUND = "Command not found"
BAD_ARG = "Bad arguments"
AMBIG = "Ambiguous command"
CMD1 = "Command 1 parsed!"
CMD2 = "Command 2 parsed!"
SINGLE = "Single word command parsed!"
SINGLE_LONG = "Single long word command parsed!"
AUTO1 = "Autocomplete command 1 parsed!"
AUTO2 = "Autocomplete command 2 parsed!"

# misc defines
CMD_QUIT = "quit"
CMD_GET_BUFSIZE = "get_history_bufsize"
BUFSIZE_TEMPLATE = "History buffer size: "
PROMPT = "CMDLINE_TEST>>"

# test defines
# each test tests progressively diverse set of keys. this way for example
# if we want to use some key sequence in the test, we first need to test
# that it itself does what it is expected to do. Most of the tests are
# designed that way.
#
# example: "arrows & delete test 1". we enter a partially valid command,
# then move 3 chars left and use delete three times. this way we get to
# know that "delete", "left" and "ctrl+B" all work (because if any of
# them fails, the whole test will fail and next tests won't be run).
#
# each test consists of name, character sequence to send to child,
# and expected output (if any).

tests = [
# test basic commands
	{"Name" : "command test 1",
	 "Sequence" : "ambiguous first" + ENTER,
	 "Result" : CMD1},
	{"Name" : "command test 2",
	 "Sequence" : "ambiguous second" + ENTER,
	 "Result" : CMD2},
	{"Name" : "command test 3",
	 "Sequence" : "ambiguous ambiguous" + ENTER,
	 "Result" : AMBIG},
	{"Name" : "command test 4",
	 "Sequence" : "ambiguous ambiguous2" + ENTER,
	 "Result" : AMBIG},

	{"Name" : "invalid command test 1",
	 "Sequence" : "ambiguous invalid" + ENTER,
	 "Result" : BAD_ARG},
# test invalid commands
	{"Name" : "invalid command test 2",
	 "Sequence" : "invalid" + ENTER,
	 "Result" : NOT_FOUND},
	{"Name" : "invalid command test 3",
	 "Sequence" : "ambiguousinvalid" + ENTER2,
	 "Result" : NOT_FOUND},

# test arrows and deletes
	{"Name" : "arrows & delete test 1",
	 "Sequence" : "singlebad" + LEFT*2 + CTRL_B + DEL*3 + ENTER,
	 "Result" : SINGLE},
	{"Name" : "arrows & delete test 2",
	 "Sequence" : "singlebad" + LEFT*5 + RIGHT + CTRL_F + DEL*3 + ENTER,
	 "Result" : SINGLE},

# test backspace
	{"Name" : "backspace test",
	 "Sequence" : "singlebad" + BKSPACE*3 + ENTER,
	 "Result" : SINGLE},

# test goto left and goto right
	{"Name" : "goto left test",
	 "Sequence" : "biguous first" + CTRL_A + "am" + ENTER,
	 "Result" : CMD1},
	{"Name" : "goto right test",
	 "Sequence" : "biguous fir" + CTRL_A + "am" + CTRL_E + "st" + ENTER,
	 "Result" : CMD1},

# test goto words
	{"Name" : "goto left word test",
	 "Sequence" : "ambiguous st" + ALT_B + "fir" + ENTER,
	 "Result" : CMD1},
	{"Name" : "goto right word test",
	 "Sequence" : "ambig first" + CTRL_A + ALT_F + "uous" + ENTER,
	 "Result" : CMD1},

# test removing words
	{"Name" : "remove left word 1",
	 "Sequence" : "single invalid" + CTRL_W + ENTER,
	 "Result" : SINGLE},
	{"Name" : "remove left word 2",
	 "Sequence" : "single invalid" + ALT_BKSPACE + ENTER,
	 "Result" : SINGLE},
	{"Name" : "remove right word",
	 "Sequence" : "single invalid" + ALT_B + ALT_D + ENTER,
	 "Result" : SINGLE},

# test kill buffer (copy and paste)
	{"Name" : "killbuffer test 1",
	 "Sequence" : "ambiguous" + CTRL_A + CTRL_K + " first" + CTRL_A + CTRL_Y + ENTER,
	 "Result" : CMD1},
	{"Name" : "killbuffer test 2",
	 "Sequence" : "ambiguous" + CTRL_A + CTRL_K + CTRL_Y*26 + ENTER,
	 "Result" : NOT_FOUND},

# test newline
	{"Name" : "newline test",
	 "Sequence" : "invalid" + CTRL_C + "single" + ENTER,
	 "Result" : SINGLE},

# test redisplay (nothing should really happen)
	{"Name" : "redisplay test",
	 "Sequence" : "single" + CTRL_L + ENTER,
	 "Result" : SINGLE},

# test autocomplete
	{"Name" : "autocomplete test 1",
	 "Sequence" : "si" + TAB + ENTER,
	 "Result" : SINGLE},
	{"Name" : "autocomplete test 2",
	 "Sequence" : "si" + TAB + "_" + TAB + ENTER,
	 "Result" : SINGLE_LONG},
	{"Name" : "autocomplete test 3",
	 "Sequence" : "in" + TAB + ENTER,
	 "Result" : NOT_FOUND},
	{"Name" : "autocomplete test 4",
	 "Sequence" : "am" + TAB + ENTER,
	 "Result" : BAD_ARG},
	{"Name" : "autocomplete test 5",
	 "Sequence" : "am" + TAB + "fir" + TAB + ENTER,
	 "Result" : CMD1},
	{"Name" : "autocomplete test 6",
	 "Sequence" : "am" + TAB + "fir" + TAB + TAB + ENTER,
	 "Result" : CMD1},
	{"Name" : "autocomplete test 7",
	 "Sequence" : "am" + TAB + "fir" + TAB + " " + TAB + ENTER,
	 "Result" : CMD1},
	{"Name" : "autocomplete test 8",
	 "Sequence" : "am" + TAB + "     am" + TAB + "   " + ENTER,
	 "Result" : AMBIG},
	{"Name" : "autocomplete test 9",
	 "Sequence" : "am" + TAB + "inv" + TAB + ENTER,
	 "Result" : BAD_ARG},
	{"Name" : "autocomplete test 10",
	 "Sequence" : "au" + TAB + ENTER,
	 "Result" : NOT_FOUND},
	{"Name" : "autocomplete test 11",
	 "Sequence" : "au" + TAB + "1" + ENTER,
	 "Result" : AUTO1},
	{"Name" : "autocomplete test 12",
	 "Sequence" : "au" + TAB + "2" + ENTER,
	 "Result" : AUTO2},
	{"Name" : "autocomplete test 13",
	 "Sequence" : "au" + TAB + "2" + TAB + ENTER,
	 "Result" : AUTO2},
	{"Name" : "autocomplete test 14",
	 "Sequence" : "au" + TAB + "2   " + TAB + ENTER,
	 "Result" : AUTO2},
	{"Name" : "autocomplete test 15",
	 "Sequence" : "24" + TAB + ENTER,
	 "Result" : "24"},

# test history
	{"Name" : "history test 1",
	 "Sequence" : "invalid" + ENTER + "single" + ENTER + "invalid" + ENTER + UP + CTRL_P + ENTER,
	 "Result" : SINGLE},
	{"Name" : "history test 2",
	 "Sequence" : "invalid" + ENTER + "ambiguous first" + ENTER + "invalid" + ENTER + "single" + ENTER + UP * 3 + CTRL_N + DOWN + ENTER,
	 "Result" : SINGLE},

#
# tests that improve coverage
#

# empty space tests
	{"Name" : "empty space test 1",
	 "Sequence" : RIGHT + LEFT + CTRL_B + CTRL_F + ENTER,
	 "Result" : PROMPT},
	{"Name" : "empty space test 2",
	 "Sequence" : BKSPACE + ENTER,
	 "Result" : PROMPT},
	{"Name" : "empty space test 3",
	 "Sequence" : CTRL_E*2 + CTRL_A*2 + ENTER,
	 "Result" : PROMPT},
	{"Name" : "empty space test 4",
	 "Sequence" : ALT_F*2 + ALT_B*2 + ENTER,
	 "Result" : PROMPT},
	{"Name" : "empty space test 5",
	 "Sequence" : " " + CTRL_E*2 + CTRL_A*2 + ENTER,
	 "Result" : PROMPT},
	{"Name" : "empty space test 6",
	 "Sequence" : " " + CTRL_A + ALT_F*2 + ALT_B*2 + ENTER,
	 "Result" : PROMPT},
	{"Name" : "empty space test 7",
	 "Sequence" : "  " + CTRL_A + CTRL_D + CTRL_E + CTRL_D + ENTER,
	 "Result" : PROMPT},
	{"Name" : "empty space test 8",
	 "Sequence" : " space" + CTRL_W*2 + ENTER,
	 "Result" : PROMPT},
	{"Name" : "empty space test 9",
	 "Sequence" : " space" + ALT_BKSPACE*2 + ENTER,
	 "Result" : PROMPT},
	{"Name" : "empty space test 10",
	 "Sequence" : " space " + CTRL_A + ALT_D*3 + ENTER,
	 "Result" : PROMPT},

# non-printable char tests
	{"Name" : "non-printable test 1",
	 "Sequence" : chr(27) + chr(47) + ENTER,
	 "Result" : PROMPT},
	{"Name" : "non-printable test 2",
	 "Sequence" : chr(27) + chr(128) + ENTER*7,
	 "Result" : PROMPT},
	{"Name" : "non-printable test 3",
	 "Sequence" : chr(27) + chr(91) + chr(127) + ENTER*6,
	 "Result" : PROMPT},

# miscellaneous tests
	{"Name" : "misc test 1",
	 "Sequence" : ENTER,
	 "Result" : PROMPT},
	{"Name" : "misc test 2",
	 "Sequence" : "single #comment" + ENTER,
	 "Result" : SINGLE},
	{"Name" : "misc test 3",
	 "Sequence" : "#empty line" + ENTER,
	 "Result" : PROMPT},
	{"Name" : "misc test 4",
	 "Sequence" : "   single  " + ENTER,
	 "Result" : SINGLE},
	{"Name" : "misc test 5",
	 "Sequence" : "single#" + ENTER,
	 "Result" : SINGLE},
	{"Name" : "misc test 6",
	 "Sequence" : 'a' * 257 + ENTER,
	 "Result" : NOT_FOUND},
	{"Name" : "misc test 7",
	 "Sequence" : "clear_history" + UP*5 + DOWN*5 + ENTER,
	 "Result" : PROMPT},
	{"Name" : "misc test 8",
	 "Sequence" : "a" + HELP + CTRL_C,
	 "Result" : PROMPT},
	{"Name" : "misc test 9",
	 "Sequence" : CTRL_D*3,
	 "Result" : None},
]



#!/usr/bin/python

#   BSD LICENSE
#
#   Copyright(c) 2010-2014 Intel Corporation. All rights reserved.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     * Neither the name of Intel Corporation nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Script that runs cmdline_test app and feeds keystrokes into it.

import sys, pexpect, string, os, cmdline_test_data

#
# function to run test
#
def runTest(child,test):
	child.send(test["Sequence"])
	if test["Result"] == None:
		return 0
	child.expect(test["Result"],1)

#
# history test is a special case
#
# This test does the following:
# 1) fills the history with garbage up to its full capacity
#    (just enough to remove last entry)
# 2) scrolls back history to the very beginning
# 3) checks if the output is as expected, that is, the first
#    number in the sequence (not the last entry before it)
#
# This is a self-contained test, it needs only a pexpect child
#
def runHistoryTest(child):
	# find out history size
	child.sendline(cmdline_test_data.CMD_GET_BUFSIZE)
	child.expect("History buffer size: \\d+", timeout=1)
	history_size = int(child.after[len(cmdline_test_data.BUFSIZE_TEMPLATE):])
	i = 0

	# fill the history with numbers
	while i < history_size / 10:
		# add 1 to prevent from parsing as octals
		child.send("1" + str(i).zfill(8) + cmdline_test_data.ENTER)
		# the app will simply print out the number
		child.expect(str(i + 100000000), timeout=1)
		i += 1
	# scroll back history
	child.send(cmdline_test_data.UP * (i + 2) + cmdline_test_data.ENTER)
	child.expect("100000000", timeout=1)

# the path to cmdline_test executable is supplied via command-line.
if len(sys.argv) < 2:
	print "Error: please supply cmdline_test app path"
	sys.exit(1)

test_app_path = sys.argv[1]

if not os.path.exists(test_app_path):
	print "Error: please supply cmdline_test app path"
	sys.exit(1)

child = pexpect.spawn(test_app_path)

print "Running command-line tests..."
for test in cmdline_test_data.tests:
	print (test["Name"] + ":").ljust(30),
	try:
		runTest(child,test)
		print "PASS"
	except:
		print "FAIL"
		print child
		sys.exit(1)

# since last test quits the app, run new instance
child = pexpect.spawn(test_app_path)

print ("History fill test:").ljust(30),
try:
	runHistoryTest(child)
	print "PASS"
except:
	print "FAIL"
	print child
	sys.exit(1)
child.close()
sys.exit(0)



#! /usr/bin/python
#
#   BSD LICENSE
#
#   Copyright(c) 2010-2014 Intel Corporation. All rights reserved.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     * Neither the name of Intel Corporation nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import sys

sockets = []
cores = []
core_map = {}

fd=open("/proc/cpuinfo")
lines = fd.readlines()
fd.close()

core_details = []
core_lines = {}
for line in lines:
	if len(line.strip()) != 0:
		name, value = line.split(":", 1)
		core_lines[name.strip()] = value.strip()
	else:
		core_details.append(core_lines)
		core_lines = {}

for core in core_details:
	for field in ["processor", "core id", "physical id"]:
		if field not in core:
			print "Error getting '%s' value from /proc/cpuinfo" % field
			sys.exit(1)
		core[field] = int(core[field])

	if core["core id"] not in cores:
		cores.append(core["core id"])
	if core["physical id"] not in sockets:
		sockets.append(core["physical id"])
	key = (core["physical id"], core["core id"])
	if key not in core_map:
		core_map[key] = []
	core_map[key].append(core["processor"])

print "============================================================"
print "Core and Socket Information (as reported by '/proc/cpuinfo')"
print "============================================================\n"
print "cores = ",cores
print "sockets = ", sockets
print ""

max_processor_len = len(str(len(cores) * len(sockets) * 2 - 1))
max_core_map_len = max_processor_len * 2 + len('[, ]') + len('Socket ')
max_core_id_len = len(str(max(cores)))

print " ".ljust(max_core_id_len + len('Core ')),
for s in sockets:
        print "Socket %s" % str(s).ljust(max_core_map_len - len('Socket ')),
print ""
print " ".ljust(max_core_id_len + len('Core ')),
for s in sockets:
        print "--------".ljust(max_core_map_len),
print ""

for c in cores:
        print "Core %s" % str(c).ljust(max_core_id_len),
        for s in sockets:
                print str(core_map[(s,c)]).ljust(max_core_map_len),
        print "\n"



#!/usr/bin/python

#   BSD LICENSE
#
#   Copyright(c) 2010-2014 Intel Corporation. All rights reserved.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     * Neither the name of Intel Corporation nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The main logic behind running autotests in parallel

import multiprocessing, sys, pexpect, time, os, StringIO, csv

# wait for prompt
def wait_prompt(child):
	try:
		child.sendline()
		result = child.expect(["RTE>>", pexpect.TIMEOUT, pexpect.EOF],
			timeout = 120)
	except:
		return False
	if result == 0:
		return True
	else:
		return False

# run a test group
# each result tuple in results list consists of:
#   result value (0 or -1)
#   result string
#   test name
#   total test run time (double)
#   raw test log
#   test report (if not available, should be None)
#
# this function needs to be outside AutotestRunner class
# because otherwise Pool won't work (or rather it will require
# quite a bit of effort to make it work).
def run_test_group(cmdline, test_group):
	results = []
	child = None
	start_time = time.time()
	startuplog = None

	# run test app
	try:
		# prepare logging of init
		startuplog = StringIO.StringIO()

		print >>startuplog, "\n%s %s\n" % ("="*20, test_group["Prefix"])

		child = pexpect.spawn(cmdline, logfile=startuplog)

		# wait for target to boot
		if not wait_prompt(child):
			child.close()

			results.append((-1, "Fail [No prompt]", "Start %s" % test_group["Prefix"],
				time.time() - start_time, startuplog.getvalue(), None))

			# mark all tests as failed
			for test in test_group["Tests"]:
				results.append((-1, "Fail [No prompt]", test["Name"],
				time.time() - start_time, "", None))
			# exit test
			return results

	except:
		results.append((-1, "Fail [Can't run]", "Start %s" % test_group["Prefix"],
				time.time() - start_time, startuplog.getvalue(), None))

		# mark all tests as failed
		for t in test_group["Tests"]:
			results.append((-1, "Fail [Can't run]", t["Name"],
				time.time() - start_time, "", None))
		# exit test
		return results

	# startup was successful
	results.append((0, "Success", "Start %s" % test_group["Prefix"],
		time.time() - start_time, startuplog.getvalue(), None))

	# run all tests in test group
	for test in test_group["Tests"]:

		# create log buffer for each test
		# in multiprocessing environment, the logging would be
		# interleaved and will create a mess, hence the buffering
		logfile = StringIO.StringIO()
		child.logfile = logfile

		result = ()

		# make a note when the test started
		start_time = time.time()

		try:
			# print test name to log buffer
			print >>logfile, "\n%s %s\n" % ("-"*20, test["Name"])

			# run test function associated with the test
			result = test["Func"](child, test["Command"])

			# make a note when the test was finished
			end_time = time.time()

			# append test data to the result tuple
			result += (test["Name"], end_time - start_time,
				logfile.getvalue())

			# call report function, if any defined, and supply it with
			# target and complete log for test run
			if test["Report"]:
				report = test["Report"](self.target, log)

				# append report to results tuple
				result += (report,)
			else:
				# report is None
				result += (None,)
		except:
			# make a note when the test crashed
			end_time = time.time()

			# mark test as failed
			result = (-1, "Fail [Crash]", test["Name"],
				end_time - start_time, logfile.getvalue(), None)
		finally:
			# append the results to the results list
			results.append(result)

	# regardless of whether test has crashed, try quitting it
	try:
		child.sendline("quit")
		child.close()
	# if the test crashed, just do nothing instead
	except:
		# nop
		pass

	# return test results
	return results





# class representing an instance of autotests run
class AutotestRunner:
	cmdline = ""
	parallel_test_groups = []
	non_parallel_test_groups = []
	logfile = None
	csvwriter = None
	target = ""
	start = None
	n_tests = 0
	fails = 0
	log_buffers = []
	blacklist = []
	whitelist = []


	def __init__(self, cmdline, target, blacklist, whitelist):
		self.cmdline = cmdline
		self.target = target
		self.blacklist = blacklist
		self.whitelist = whitelist

		# log file filename
		logfile = "%s.log" % target
		csvfile = "%s.csv" % target

		self.logfile = open(logfile, "w")
		csvfile = open(csvfile, "w")
		self.csvwriter = csv.writer(csvfile)

		# prepare results table
		self.csvwriter.writerow(["test_name","test_result","result_str"])



	# set up cmdline string
	def __get_cmdline(self, test):
		cmdline = self.cmdline

		# append memory limitations for each test
		# otherwise tests won't run in parallel
		if not "i686" in self.target:
			cmdline += " --socket-mem=%s"% test["Memory"]
		else:
			# affinitize startup so that tests don't fail on i686
			cmdline = "taskset 1 " + cmdline
			cmdline += " -m " + str(sum(map(int,test["Memory"].split(","))))

		# set group prefix for autotest group
		# otherwise they won't run in parallel
		cmdline += " --file-prefix=%s"% test["Prefix"]

		return cmdline



	def add_parallel_test_group(self,test_group):
		self.parallel_test_groups.append(test_group)

	def add_non_parallel_test_group(self,test_group):
		self.non_parallel_test_groups.append(test_group)


	def __process_results(self, results):
		# this iterates over individual test results
		for i, result in enumerate(results):

			# increase total number of tests that were run
			# do not include "start" test
			if i > 0:
				self.n_tests += 1

			# unpack result tuple
			test_result, result_str, test_name, \
				test_time, log, report = result

			# get total run time
			cur_time = time.time()
			total_time = int(cur_time - self.start)

			# print results, test run time and total time since start
			print ("%s:" % test_name).ljust(30),
			print result_str.ljust(29),
			print "[%02dm %02ds]" % (test_time / 60, test_time % 60),

			# don't print out total time every line, it's the same anyway
			if i == len(results) - 1:
				print "[%02dm %02ds]" % (total_time / 60, total_time % 60)
			else:
				print ""

			# if test failed and it wasn't a "start" test
			if test_result < 0 and not i == 0:
				self.fails += 1

			# collect logs
			self.log_buffers.append(log)

			# create report if it exists
			if report:
				try:
					f = open("%s_%s_report.rst" % (self.target,test_name), "w")
				except IOError:
					print "Report for %s could not be created!" % test_name
				else:
					with f:
						f.write(report)

			# write test result to CSV file
			if i != 0:
				self.csvwriter.writerow([test_name, test_result, result_str])




	# this function iterates over test groups and removes each
	# test that is not in whitelist/blacklist
	def __filter_groups(self, test_groups):
		groups_to_remove = []

		# filter out tests from parallel test groups
		for i, test_group in enumerate(test_groups):

			# iterate over a copy so that we could safely delete individual tests
			for test in test_group["Tests"][:]:
				test_id = test["Command"]

				# dump tests are specified in full e.g. "Dump_mempool"
				if "_autotest" in test_id:
					test_id = test_id[:-len("_autotest")]

				# filter out blacklisted/whitelisted tests
				if self.blacklist and test_id in self.blacklist:
					test_group["Tests"].remove(test)
					continue
				if self.whitelist and test_id not in self.whitelist:
					test_group["Tests"].remove(test)
					continue

			# modify or remove original group
			if len(test_group["Tests"]) > 0:
				test_groups[i] = test_group
			else:
				# remember which groups should be deleted
				# put the numbers backwards so that we start
				# deleting from the end, not from the beginning
				groups_to_remove.insert(0, i)

		# remove test groups that need to be removed
		for i in groups_to_remove:
			del test_groups[i]

		return test_groups



	# iterate over test groups and run tests associated with them
	def run_all_tests(self):
		# filter groups
		self.parallel_test_groups = \
			self.__filter_groups(self.parallel_test_groups)
		self.non_parallel_test_groups = \
			self.__filter_groups(self.non_parallel_test_groups)

		# create a pool of worker threads
		pool = multiprocessing.Pool(processes=1)

		results = []

		# whatever happens, try to save as much logs as possible
		try:

			# create table header
			print ""
			print "Test name".ljust(30),
			print "Test result".ljust(29),
			print "Test".center(9),
			print "Total".center(9)
			print "=" * 80

			# make a note of tests start time
			self.start = time.time()

			# assign worker threads to run test groups
			for test_group in self.parallel_test_groups:
				result = pool.apply_async(run_test_group,
					[self.__get_cmdline(test_group), test_group])
				results.append(result)

			# iterate while we have group execution results to get
			while len(results) > 0:

				# iterate over a copy to be able to safely delete results
				# this iterates over a list of group results
				for group_result in results[:]:

					# if the thread hasn't finished yet, continue
					if not group_result.ready():
						continue

					res = group_result.get()

					self.__process_results(res)

					# remove result from results list once we're done with it
					results.remove(group_result)

			# run non_parallel tests. they are run one by one, synchronously
			for test_group in self.non_parallel_test_groups:
				group_result = run_test_group(self.__get_cmdline(test_group), test_group)

				self.__process_results(group_result)

			# get total run time
			cur_time = time.time()
			total_time = int(cur_time - self.start)

			# print out summary
			print "=" * 80
			print "Total run time: %02dm %02ds" % (total_time / 60, total_time % 60)
			if self.fails != 0:
				print "Number of failed tests: %s" % str(self.fails)

			# write summary to logfile
			self.logfile.write("Summary\n")
			self.logfile.write("Target: ".ljust(15) + "%s\n" % self.target)
			self.logfile.write("Tests: ".ljust(15) + "%i\n" % self.n_tests)
			self.logfile.write("Failed tests: ".ljust(15) + "%i\n" % self.fails)
		except:
			print "Exception occured"
			print sys.exc_info()
			self.fails = 1

		# drop logs from all executions to a logfile
		for buf in self.log_buffers:
			self.logfile.write(buf.replace("\r",""))

		log_buffers = []

		return self.fails
